@@ -360,7 +360,6 @@ private:
 	struct LogBuffer {
 		// Indexed by topic string.
 		std::unordered_map<std::string, broker::vector> msgs;
-		double last_flush;
 		size_t message_count;
 
 		size_t Flush(broker::endpoint& endpoint, size_t batch_size);
@@ -396,7 +395,6 @@ private:
 	int times_processed_without_idle;
 
 	size_t log_batch_size;
-	double log_batch_interval;
 	Func* log_topic_func;
 	VectorType* vector_of_data_type;
 	EnumType* log_id_type;
